﻿using UnityEngine;
using FMODUnity;
using HarmonyLib;
using Steamworks;
using System;
using System.Collections.Generic;
using System.Linq;


public class Shared_Inventory : Mod
{
	public void BuildFilterSettings()
    {
        if(itemFilter_Seeds != null)
			itemFilter_Seeds.Clear();
		itemFilter_Seeds = new List<int>();
		if(itemFilter_Arrow != null)
			itemFilter_Arrow.Clear();
		itemFilter_Arrow = new List<int>();
		//seeds
		itemFilter_Seeds.Add(330);
		itemFilter_Seeds.Add(284);
		itemFilter_Seeds.Add(119);
		itemFilter_Seeds.Add(123);
		itemFilter_Seeds.Add(121);
		itemFilter_Seeds.Add(120);
		itemFilter_Seeds.Add(122);
		itemFilter_Seeds.Add(236);
		itemFilter_Seeds.Add(167);
		itemFilter_Seeds.Add(31);
		itemFilter_Seeds.Add(283);
		itemFilter_Seeds.Add(168);
		itemFilter_Seeds.Add(329);
		itemFilter_Seeds.Add(166);
		//Ammo
		itemFilter_Arrow.Add(207);
		itemFilter_Arrow.Add(186);
		itemFilter_Arrow.Add(557);
	}
	public bool canMove(Item_Base ib, PlayerInventory pinv)
	{
		if(ib == null || pinv == null)
		{
			UnityEngine.Debug.Log("canMove => ib or pinv null!");
			return false;
		}
		if(ignoreCheck)
			return ignoreCheck;
		if(itemFilter_Seeds.Contains(ib.UniqueIndex))
		{
			UnityEngine.Debug.Log("canMove => is seed!");
			return false;
		}
		if(pinv.GetItemCount("Bow") >= 1 && itemFilter_Arrow.Contains(ib.UniqueIndex))
		{
			UnityEngine.Debug.Log("canMove => has Bow!");
			UnityEngine.Debug.Log("canMove => is ammo => " + ib.settings_Inventory.DisplayName);
			return false;
		}
		if(pinv.GetItemCount("NetGun") >= 1 && ib.UniqueIndex == 245)
		{
			UnityEngine.Debug.Log("canMove => has NetGun!");
			UnityEngine.Debug.Log("canMove => is ammo => " + ib.settings_Inventory.DisplayName);
			return false;
		}
		if(ib.settings_Inventory.StackSize == 1 && isUniqueTools(ib))
		{
			UnityEngine.Debug.Log("canMove => is UniqueTools and StackSize == 1!");
			return false;
		}
		if(isAdvanceEdible(ib))
		{
			UnityEngine.Debug.Log("canMove => is advance edible item!");
			return false;
		}
		return true;
	}
	public bool isAdvanceEdible(Item_Base ib)
	{
		if(ib.settings_recipe == null)
			return false;
		if(ib.settings_consumeable == null)
			return false;
		ItemInstance_Consumeable EdibleItem = ib.settings_consumeable;
		if(EdibleItem.OxygenYield == 0f && EdibleItem.HungerYield == 0f && EdibleItem.ThirstYield == 0f && EdibleItem.BonusHungerYield == 0f && EdibleItem.BonusThirstYield == 0f)
			return false;
		return true;
	}
	public bool isUniqueTools(Item_Base ib)
	{
		
		return ib.settings_recipe.CraftingCategory == CraftingCategory.Tools || ib.settings_recipe.CraftingCategory == CraftingCategory.Weapons || ib.settings_recipe.CraftingCategory == CraftingCategory.Equipment || ib.settings_recipe.CraftingCategory == CraftingCategory.Navigation;
		
	}
	
	public List<int> itemFilter_Seeds;
	public List<int> itemFilter_Arrow;
	
	
    public void Start()
    {
        Debug.Log("Mod Shared_Inventory has been loaded!");
		BuildFilterSettings();
    }

    public void OnModUnload()
    {
        Debug.Log("Mod Shared_Inventory has been unloaded!");
    }

	public override void WorldEvent_WorldLoaded()
	{
		UnityEngine.Debug.Log("WorldEvent_WorldLoaded => first rebuild!");

	}
	public override void LocalPlayerEvent_PickupItem(PickupItem item)
	{
		UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => pickSomething!");
		if(item.itemInstance != null && item.itemInstance.baseItem != null && item.itemInstance.baseItem.settings_Inventory != null)
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => itemInstance => " + item.itemInstance.baseItem.settings_Inventory.DisplayName);
		else
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => no itemInstance");
		
		if(item.dropper != null)
		{
			Item_Base[] randomItems = item.dropper.GetRandomItems();
			for (int i = 0; i < randomItems.Length; i++)
			{
				if (randomItems[i] != null)
				{
					UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => dropper => " + randomItems[i].settings_Inventory.DisplayName);
				}
			}
		}
		else
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => dropper is null");
		
		Cost cost = null;
		try
		{
			cost = item.yieldHandler.Yield.Last();
		}catch(Exception e)
		{
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => Yield.Last is null");
		}
		if (cost == null || cost.item == null)
		{
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => Yield is null");
		}
		else
		{
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => cost => " + cost.item.settings_Inventory.DisplayName);
			UnityEngine.Debug.Log("LocalPlayerEvent_PickupItem => cost amount => " + cost.amount.ToString());
		}
		
		moveItem();
	}
	public void	 moveItem()
	{
		UnityEngine.Debug.Log("Shared_Inventory => start moveItem!!");
		 Network_Player player = RAPI.GetLocalPlayer();
		 if(player == null)
		 {
			UnityEngine.Debug.Log("Shared_Inventory => player == null!!");
			return;
		 }
		 PlayerInventory playerInventory = player.Inventory;
		 if(playerInventory == null)
		 {
			UnityEngine.Debug.Log("Shared_Inventory => playerInventory == null!!");
			return;
		 }
		 
		 foreach(Storage_Small storage in StorageManager.allStorages)
		{
			UnityEngine.Debug.Log("Shared_Inventory => one storage");
			Inventory inv = storage.GetInventoryReference();
			
			if (storage.IsOpen || inv == null || inv == playerInventory)
				continue;
			inv.secondInventory = playerInventory;
			SortPlayerInventoryIntoStorage(playerInventory, inv);

		}
	}
	public void SortPlayerInventoryIntoStorage(PlayerInventory sec, Inventory targetInv)
	{
		targetInv.secondInventory = sec;
		if (targetInv.secondInventory == null || !(targetInv.secondInventory is PlayerInventory))
		{
			return;
		}
		Dictionary<int, int> dictionary = new Dictionary<int, int>();
		for (int i = 0; i < targetInv.allSlots.Count; i++)
		{
			
			Slot slot = targetInv.allSlots[i];
			if (!(slot == null) && slot.HasValidItemInstance())
			{
				int uniqueIndex = slot.itemInstance.baseItem.UniqueIndex;
				if (slot.itemInstance.HasMaxUses && slot.itemInstance.Amount <= slot.itemInstance.baseItem.settings_Inventory.StackSize && (slot.itemInstance.BaseItemMaxUses <= 1 || slot.itemInstance.settings_Inventory.StackSize != 1))
				{
					if (dictionary.ContainsKey(uniqueIndex))
					{
						Dictionary<int, int> dictionary2 = dictionary;
						int key = uniqueIndex;
						dictionary2[key] += slot.itemInstance.Amount;
					}
					else
					{
						dictionary.Add(uniqueIndex, slot.itemInstance.Amount);
					}
				}
			}
		}
		Dictionary<int, int> dictionary3 = new Dictionary<int, int>();
		for (int j = 0; j < targetInv.secondInventory.allSlots.Count; j++)
		{
			Slot slot2 = targetInv.secondInventory.allSlots[j];
			if (!(slot2 == null) && slot2.HasValidItemInstance())
			{
				int uniqueIndex2 = slot2.itemInstance.baseItem.UniqueIndex;
				if (slot2.itemInstance.HasMaxUses && slot2.itemInstance.Amount <= slot2.itemInstance.baseItem.settings_Inventory.StackSize && (slot2.itemInstance.BaseItemMaxUses <= 1 || slot2.itemInstance.settings_Inventory.StackSize != 1))
				{
					foreach (KeyValuePair<int, int> keyValuePair in dictionary)
					{
						if (keyValuePair.Key == uniqueIndex2)
						{
							if (dictionary3.ContainsKey(uniqueIndex2))
							{
								Dictionary<int, int> dictionary2 = dictionary3;
								int key = uniqueIndex2;
								dictionary2[key] += slot2.itemInstance.Amount;
							}
							else
							{
								dictionary3.Add(uniqueIndex2, slot2.itemInstance.Amount);
							}
						}
					}
				}
			}
		}
		foreach (KeyValuePair<int, int> keyValuePair2 in dictionary3)
		{
			Item_Base itemByIndex = ItemManager.GetItemByIndex(keyValuePair2.Key);
			int num = Mathf.FloorToInt((float)keyValuePair2.Value / (float)itemByIndex.settings_Inventory.StackSize);
			int num2 = keyValuePair2.Value % itemByIndex.settings_Inventory.StackSize;
			for (int k = 0; k < num; k++)
			{
				if(!canMove(itemByIndex, sec))
					continue;
				int num3 = targetInv.AddItem(itemByIndex.UniqueName, itemByIndex.settings_Inventory.StackSize);
				targetInv.secondInventory.RemoveItem(itemByIndex.UniqueName, itemByIndex.settings_Inventory.StackSize - num3);
			}
			if (num2 > 0)
			{
				if(!canMove(itemByIndex, sec))
					continue;
				int num4 = targetInv.AddItem(itemByIndex.UniqueName, num2);
				targetInv.secondInventory.RemoveItem(itemByIndex.UniqueName, num2 - num4);
			}
		}
	}
	
	//mod settings
	public bool ignoreCheck = false;
	public void ExtraSettingsAPI_Load()
    {
		BuildSetting();
	}
	public void ExtraSettingsAPI_SettingsClose()
    {
		BuildSetting();
	}
	public void BuildSetting()
	{
		if (!ExtraSettingsAPI_Loaded)
        {
            UnityEngine.Debug.Log("BuildSetting failed!");
            return;
        }
		ignoreCheck = ExtraSettingsAPI_GetCheckboxState("ignore stackable item check");
		UnityEngine.Debug.Log("BuildSetting => ignoreCheck => " + ignoreCheck.ToString());
	}
	public static bool ExtraSettingsAPI_GetCheckboxState(string SettingName) => new bool();
	static bool ExtraSettingsAPI_Loaded = false;
}